from api import utils
from copy import copy
from decimal import Decimal, InvalidOperation
from optparse import *

def check_positive_decimal(option, opt, value):
	try:
		d = Decimal(value)
	except InvalidOperation:
		raise OptionValueError('option %s: invalid decimal value: %s' % (opt, value))

	if d >= 0 and d <= 1000000:
		return d
	else:
		raise OptionValueError('option %s: expected value (0 <= x <= 1000000), got %s instead' % (opt, d))

def check_positive_integer(option, opt, value):
	try:
		i = int(value)
	except ValueError:
		raise OptionValueError('option %s: invalid integer value: %s' % (opt, value))

	if i >= 0:
		return i
	else:
		raise OptionValueError('option %s: expected positive value, got %d instead' % (opt, i))

def check_rank(option, opt, value):
	try:
		return utils.get_rank(value)
	except:
		raise OptionValueError('option %s: invalid rank: %s' % (opt, value))

def check_quality(option, opt, value):
	try:
		return utils.get_quality(value)
	except:
		raise OptionValueError('option %s: invalid quality: %s' % (opt, value))

def check_industry(option, opt, value):
	try:
		return utils.get_industry(value)
	except:
		raise OptionValueError('option %s: invalid industry: %s' % (opt, value))

def check_domain(option, opt, value):
	try:
		return utils.get_domain(value)
	except:
		raise OptionValueError('option %s: invalid domain: %s' % (opt, value))

class ErepublikParserOption(Option):
	TYPES = Option.TYPES + ('+decimal', '+integer', 'rank', 'quality', 'wellness', 'happiness', 'industry', 'domain')
	TYPE_CHECKER = copy(Option.TYPE_CHECKER)
	TYPE_CHECKER['+decimal'] = check_positive_decimal
	TYPE_CHECKER['+integer'] = check_positive_integer
	TYPE_CHECKER['rank'] = check_rank
	TYPE_CHECKER['quality'] = check_quality
	TYPE_CHECKER['wellness'] = check_positive_decimal
	TYPE_CHECKER['happiness'] = check_positive_decimal
	TYPE_CHECKER['industry'] = check_industry
	TYPE_CHECKER['domain'] = check_domain

class ErepublikParserError(Exception):
	def __init__(self, message):
		self.msg = message

	def __str__(self):
		return str(self.msg)

class ErepublikParser(OptionParser):
	def error(self, error):
		raise ErepublikParserError(error)
