import urllib
from datetime import datetime
from feed import get_json

class Weather(object):
	def __init__(self, key):
		self.API_KEY = key
		self.cache = {}
		self.last_min_requests = []
		self.REQ_LIMIT = 99
	
	def api_request(self, url):
		self.last_min_requests = [req for req in self.last_min_requests if (datetime.now() - req).seconds < 60]
		
		if len(self.last_min_requests) >= self.REQ_LIMIT:
			raise WeatherException('weather data is temporarily unavailable. Try again later')
		
		self.last_min_requests.append(datetime.now())
		return get_json(url)
	
	def get_conditions(self, location):
		if location.lower() in self.cache or location.startswith('/q/zmw'):
			zmw = self.cache.get(location.lower(), location)
			js = self.api_request('http://api.wunderground.com/api/{key}/conditions{zmw}.json'.format(key=self.API_KEY, zmw=zmw))
			return js['current_observation']

		js = self.api_request(u'http://api.wunderground.com/api/{key}/geolookup/conditions/q/{loc}.json'.format(key=self.API_KEY, loc=urllib.quote(location)))
		if 'current_observation' in js: # Only 1 city found
			zmw = js['location']['l']
		else:
			zmw = self.find_zmw(js)
			if zmw and 'l' in zmw:
				zmw = zmw['l']
			else:
				raise WeatherException('No cities match your search query')
			# request weather conditions for this location
			js = self.api_request('http://api.wunderground.com/api/{key}/conditions{zmw}.json'.format(key=self.API_KEY, zmw=zmw))
			
		if 'current_observation' not in js:
			raise WeatherException('No cities match your search query')
		
		self.cache[location.lower()] = zmw
		return js['current_observation']
	
	def forecast(self, location):
		if location.lower() in self.cache or location.startswith('/q/zmw'):
			zmw = self.cache.get(location.lower(), location)
			js = self.api_request('http://api.wunderground.com/api/{key}/geolookup/forecast{zmw}.json'.format(key=self.API_KEY, zmw=zmw))
			return js['location'], js['forecast']

		js = self.api_request(u'http://api.wunderground.com/api/{key}/geolookup/forecast/q/{loc}.json'.format(key=self.API_KEY, loc=urllib.quote(location)))
		if 'forecast' in js: # Only 1 city found
			zmw = js['location']['l']
		else:
			zmw = self.find_zmw(js)
			if zmw and 'l' in zmw:
				zmw = zmw['l']
			else:
				raise WeatherException('No cities match your search query')
			# request weather conditions for this location
			js = self.api_request('http://api.wunderground.com/api/{key}/geolookup/forecast{zmw}.json'.format(key=self.API_KEY, zmw=zmw))
		
		if 'location' not in js and 'forecast' not in js:
			raise WeatherException('No cities match your search query')
		
		self.cache[location.lower()] = zmw
		return js['location'], js['forecast']
	
	def find_zmw(self, js):
		if isinstance(js, basestring):
			js = self.api_request(u'http://api.wunderground.com/api/{key}/geolookup/q/{loc}.json'.format(key=self.API_KEY, loc=urllib.quote(js)))
			
		if 'location' in js:
			return js['location']
		elif 'results' in js['response']: # More than 1 city found,
				loc_data = js['response']['results'][0] # choose the first result
				for loc in js['response']['results'][1:]:
					if loc['country_name'] != 'USA':
						loc_data = loc # replace it with the first international city (if found)
						break
				
				return loc_data
		elif 'error' in js['response']: # nothing found
			raise WeatherException(js['response']['error']['description'])

class WeatherException(Exception):
	def __init__(self, msg):
		if msg == 'Search not found': 
			msg = 'location not found'
		self.msg = msg
	
	def __str__(self):
		return str(self.msg)
